"use strict";

import * as THREE from "three";
import { OBJLoader } from "three/addons/loaders/OBJLoader.js";


var camera, scene, renderer;
var cameraControls;
var clock = new THREE.Clock();
var skull;
var candle, candleHolder, flame; 
var backgroundMesh; 

import { OrbitControls } from "three/addons/controls/OrbitControls.js";

function fillScene() {
    scene = new THREE.Scene();
    createGradientBackground();

    var mainLight = new THREE.DirectionalLight(0xFFAA55, 0.5);
    mainLight.position.set(-200, 200, 100);
    mainLight.castShadow = true;
    mainLight.shadow.mapSize.width = 2048;
    mainLight.shadow.mapSize.height = 2048;
    mainLight.shadow.camera.near = 0.5;
    mainLight.shadow.camera.far = 1000;
    mainLight.shadow.camera.left = -500;
    mainLight.shadow.camera.right = 500;
    mainLight.shadow.camera.top = 500;
    mainLight.shadow.camera.bottom = -500;
    mainLight.shadow.bias = -0.0005;
    scene.add(mainLight);

    const cubeTextureLoader = new THREE.CubeTextureLoader();

    var ambientLight = new THREE.AmbientLight(0x222222);
    scene.add(ambientLight);

    createTable();
    loadSkull();
    createEnhancedCandle();
    loadFlower();
}

function createGradientBackground() {
    const bgGeometry = new THREE.PlaneGeometry(2000, 1000);
    
    const canvas = document.createElement('canvas');
    canvas.width = 512;
    canvas.height = 512;
    const context = canvas.getContext('2d');
   
    context.fillStyle = '#000000';
    context.fillRect(0, 0, 512, 512);
    
    const texture = new THREE.CanvasTexture(canvas);
    
    const bgMaterial = new THREE.MeshBasicMaterial({
        map: texture,
        side: THREE.DoubleSide
    });
    
    backgroundMesh = new THREE.Mesh(bgGeometry, bgMaterial);
    backgroundMesh.position.z = -400;  
    backgroundMesh.position.y = 50;    
    
    scene.add(backgroundMesh);
}

function createTable() {
   
    const tableGroup = new THREE.Group();
    

    const tableWidth = 600;
    const tableDepth = 300;
    const tableThickness = 20;
    const sideHeight = 100; 
    const panelThickness = 10; 
    
  
    const textureLoader = new THREE.TextureLoader();
    const woodTexture = textureLoader.load(
        'textures/dark_wood.jpg',
        undefined,
        undefined,
        function(error) {
            console.error('Error loading texture:', error);
        }
    ); 

    // Matériau pour toute la table
    var tableMaterial = new THREE.MeshStandardMaterial({ 
        map: woodTexture,
        roughness: 0.6,   
        metalness: 0.2    
    });

    // Plateau de la table (position inchangée)
    const tableTopGeometry = new THREE.BoxGeometry(tableWidth, tableThickness, tableDepth);
    const tableTop = new THREE.Mesh(tableTopGeometry, tableMaterial);
    tableTop.position.y = 0; 
    tableTop.receiveShadow = true; 
    tableGroup.add(tableTop);
    
    // Panneau latéral gauche (placé sous le plateau)
    const leftPanelGeometry = new THREE.BoxGeometry(panelThickness, sideHeight, tableDepth);
    const leftPanel = new THREE.Mesh(leftPanelGeometry, tableMaterial);
    leftPanel.position.set(-(tableWidth/2 - panelThickness/2), -(sideHeight/2 + tableThickness/2), 0);
    leftPanel.receiveShadow = true;
    leftPanel.castShadow = true;
    tableGroup.add(leftPanel);
    
    // Panneau latéral droit
    const rightPanelGeometry = new THREE.BoxGeometry(panelThickness, sideHeight, tableDepth);
    const rightPanel = new THREE.Mesh(rightPanelGeometry, tableMaterial);
    rightPanel.position.set((tableWidth/2 - panelThickness/2), -(sideHeight/2 + tableThickness/2), 0);
    rightPanel.receiveShadow = true;
    rightPanel.castShadow = true;
    tableGroup.add(rightPanel);
    
    // Panneau avant
    const frontPanelGeometry = new THREE.BoxGeometry(tableWidth - panelThickness*2, sideHeight, panelThickness);
    const frontPanel = new THREE.Mesh(frontPanelGeometry, tableMaterial);
    frontPanel.position.set(0, -(sideHeight/2 + tableThickness/2), (tableDepth/2 - panelThickness/2));
    frontPanel.receiveShadow = true;
    frontPanel.castShadow = true;
    tableGroup.add(frontPanel);
    
  
    const backPanelGeometry = new THREE.BoxGeometry(tableWidth - panelThickness*2, sideHeight, panelThickness);
    const backPanel = new THREE.Mesh(backPanelGeometry, tableMaterial);
    backPanel.position.set(0, -(sideHeight/2 + tableThickness/2), -(tableDepth/2 - panelThickness/2));
    backPanel.receiveShadow = true;
    backPanel.castShadow = true;
    tableGroup.add(backPanel);
    
   
    const bottomPanelGeometry = new THREE.BoxGeometry(tableWidth - panelThickness*2, panelThickness, tableDepth - panelThickness*2);
    const bottomPanel = new THREE.Mesh(bottomPanelGeometry, tableMaterial);
    bottomPanel.position.set(0, -(sideHeight + tableThickness/2), 0);
    bottomPanel.receiveShadow = true;
    bottomPanel.castShadow = true;
    tableGroup.add(bottomPanel);
    
    scene.add(tableGroup);
}
function loadSkull() {
    var loader = new OBJLoader();
    loader.load(
        'skul.obj',
        function(object) {
            skull = object;
            skull.scale.set(6, 6, 6);
            skull.position.set(120, -35, 0);
        
            skull.rotation.set(-Math.PI / 2, 0, -Math.PI / 5);
            
            const skullMaterial = new THREE.MeshPhongMaterial({  
                color: 0xFFF8F0,      
                specular: 0xFFF8F0,   
                shininess: 40,        
                emissive: 0x332211,   
                emissiveIntensity: 0.1 
            });
            
            skull.traverse(function(child) {
                if (child.isMesh) {
                    child.material = skullMaterial;
                    child.castShadow = true; 
                    child.receiveShadow = true; 
                }
            });
            scene.add(skull);
        }
    );
}
function createGradientTexture(color1, color2) {
    const canvas = document.createElement('canvas');
    canvas.width = 256;
    canvas.height = 256;
    const context = canvas.getContext('2d');

    const gradient = context.createLinearGradient(0, 0, canvas.width, canvas.height);
    gradient.addColorStop(0, color1);
    gradient.addColorStop(1, color2);

    context.fillStyle = gradient;
    context.fillRect(0, 0, canvas.width, canvas.height);

   

    const texture = new THREE.CanvasTexture(canvas);
    texture.needsUpdate = true;

    return texture;
}

function loadFlower() {
    var loader = new OBJLoader();
    loader.load(
        'fleur.obj', 
        function(object) {
            const flower = object;
            flower.scale.set(4, 4, 6); 
            flower.position.set(160, 3, 120); 

            flower.rotation.set(-Math.PI * 0.65, -Math.PI * 0.44, Math.PI * 0.05); // rotation.z pour l'inclinaison horizontale

            const gradientTexture = createGradientTexture('#ff007f', '#ff99cc');

            const flowerMaterial = new THREE.MeshStandardMaterial({
                map: gradientTexture, 
                transparent: true,

               
                side: THREE.DoubleSide 
            });

            flower.traverse(function(child) {
                if (child.isMesh) {
                    child.material = flowerMaterial;
                    child.castShadow = true;
                    child.receiveShadow = true;
                }
            });

            scene.add(flower);
            console.log("Flower added to scene at position:", flower.position);
        },
        function(xhr) {
            console.log((xhr.loaded / xhr.total * 100) + '% loaded');
        },
        function(error) {
            console.error('Error loading fleur.obj:', error);
        }
    );
}


function createEnhancedCandle() {

    const candleGroup = new THREE.Group();
   
    const holderBaseRadius = 22; 
    const holderBaseHeight = 8;  
    const holderBaseGeometry = new THREE.CylinderGeometry(holderBaseRadius, holderBaseRadius * 1.3, holderBaseHeight, 32);
    
   
    const holderStemRadius = 4.5; 
    const holderStemHeight = 25;  
    const holderStemGeometry = new THREE.CylinderGeometry(holderStemRadius * 0.8, holderStemRadius * 1.2, holderStemHeight, 16);
    
    
    const holderTopRadius = 16; 
    const holderTopHeight = 4;  
    const holderTopGeometry = new THREE.CylinderGeometry(holderTopRadius, holderTopRadius * 0.85, holderTopHeight, 32);
    
    
    const decorRadius = 18;
    const decorHeight = 1.5;
    const decorGeometry = new THREE.TorusGeometry(decorRadius * 0.7, 1.5, 12, 48);
    
    const smallDecorRadius = 10;
    const smallDecorGeometry = new THREE.TorusGeometry(smallDecorRadius * 0.7, 1, 8, 32);
    
    
    const holderMaterial = new THREE.MeshStandardMaterial({
        color: 0xFFF8F0, 
                specular: 0xFFF8F0,     
        roughness: 0.2,     
        metalness: 0.5,      
        envMapIntensity: 1   
    });
    
    // Variante plus foncée et plus détaillée
    const holderDarkMaterial = new THREE.MeshStandardMaterial({
        color: 0xFFD700,     
        roughness: 0.25,    
        metalness: 0.8,      
        envMapIntensity: 1
    });
    
    // Assemblage du chandelier
    const holderBase = new THREE.Mesh(holderBaseGeometry, holderMaterial);
    holderBase.position.y = holderBaseHeight / 2;
    holderBase.castShadow = true;  
    holderBase.receiveShadow = true;  
    
    const holderStem = new THREE.Mesh(holderStemGeometry, holderDarkMaterial);
    holderStem.position.y = holderBaseHeight + holderStemHeight / 2;
    holderStem.castShadow = true;  
    holderStem.receiveShadow = true;  
    
    const holderTop = new THREE.Mesh(holderTopGeometry, holderMaterial);
    holderTop.position.y = holderBaseHeight + holderStemHeight + holderTopHeight / 2;
    holderTop.castShadow = true;  
    holderTop.receiveShadow = true;  
    
    const decor = new THREE.Mesh(decorGeometry, holderDarkMaterial);
    decor.position.y = holderBaseHeight + 6;
    decor.rotation.x = Math.PI / 2;
    decor.castShadow = true; 
    decor.receiveShadow = true;  
    
    // Décoration supplémentaire au milieu du stem
    const middleDecor = new THREE.Mesh(smallDecorGeometry, holderMaterial);
    middleDecor.position.y = holderBaseHeight + holderStemHeight / 2;
    middleDecor.rotation.x = Math.PI / 2;
    middleDecor.castShadow = true;  
    middleDecor.receiveShadow = true;  
    
   
    const candleRadius = 5; 
    const candleHeight = 20; 
    const candleGeometry = new THREE.CylinderGeometry(candleRadius * 0.9, candleRadius, candleHeight, 32);
    
    // Matériau de cire pour la bougie plus réaliste
    const candleMaterial = new THREE.MeshStandardMaterial({
        color: 0xf3e5ab,
        roughness: 0.8,
        metalness: 0.05,
        transparent: true,
        opacity: 0.95 
    });
    
  
    const meltedWaxGeometry = new THREE.SphereGeometry(candleRadius * 1.2, 16, 16, 0, Math.PI * 2, 0, Math.PI / 2);
    meltedWaxGeometry.translate(0, -candleHeight/2, 0);
    const meltedWaxMaterial = new THREE.MeshStandardMaterial({
        color: 0xf3e5ab,
        roughness: 0.6,
        metalness: 0.1,
        transparent: true,
        opacity: 0.9
    });
    
    const meltedWax = new THREE.Mesh(meltedWaxGeometry, meltedWaxMaterial);
    meltedWax.castShadow = true;  // Permet à la cire fondue de projeter une ombre
    meltedWax.receiveShadow = true;  // Permet à la cire fondue de recevoir une ombre
    
    candle = new THREE.Mesh(candleGeometry, candleMaterial);
    candle.position.y = holderBaseHeight + holderStemHeight + holderTopHeight + candleHeight / 2 - 2; // Ajusté pour s'enfoncer légèrement
    candle.castShadow = true;
    candle.receiveShadow = true;  
    candle.add(meltedWax);
    
    // Création d'une mèche plus détaillée
    const wickRadius = 0.4;
    const wickHeight = 3;
    const wickGeometry = new THREE.CylinderGeometry(wickRadius * 0.6, wickRadius, wickHeight, 8);
    
    const wickMaterial = new THREE.MeshStandardMaterial({
        color: 0x442200,
        roughness: 1.0,
        metalness: 0.0,
        emissive: 0x221100,
        emissiveIntensity: 0.2
    });
    
    const wick = new THREE.Mesh(wickGeometry, wickMaterial);
    wick.position.y = holderBaseHeight + holderStemHeight + holderTopHeight + candleHeight + wickHeight / 2 - 2;
    wick.castShadow = true;  
    wick.receiveShadow = true;  
    
   
    const innerFlameHeight = 8;
    const innerFlameRadius = 1.5;
    const innerFlameGeometry = new THREE.ConeGeometry(innerFlameRadius, innerFlameHeight, 16, 5, true);
    
    const innerFlameMaterial = new THREE.MeshBasicMaterial({
        color: 0xffffff,      // Blanc plus pur au centre
        transparent: true,
        opacity: 0.95,
        side: THREE.DoubleSide,
        blending: THREE.AdditiveBlending
    });
    
    const innerFlame = new THREE.Mesh(innerFlameGeometry, innerFlameMaterial);
    innerFlame.position.y = holderBaseHeight + holderStemHeight + holderTopHeight + candleHeight + wickHeight + innerFlameHeight / 2 - 2;
  
    const outerFlameHeight = 12;
    const outerFlameRadius = 2.5;
    
    const outerFlameGeometry = new THREE.ConeGeometry(outerFlameRadius, outerFlameHeight, 20, 8, true);
   
    const positionAttribute = outerFlameGeometry.getAttribute('position');
    for (let i = 0; i < positionAttribute.count; i++) {
        const x = positionAttribute.getX(i);
        const y = positionAttribute.getY(i);
        const z = positionAttribute.getZ(i);
        
        
        if (y > outerFlameHeight * 0.3) {
            const distortFactor = (y / outerFlameHeight) * 0.4;
            positionAttribute.setX(i, x + (Math.random() - 0.5) * distortFactor);
            positionAttribute.setZ(i, z + (Math.random() - 0.5) * distortFactor);
        }
    }
    outerFlameGeometry.attributes.position.needsUpdate = true;
    
    // Matériau amélioré pour la flamme extérieure
    const outerFlameMaterial = new THREE.MeshBasicMaterial({
        color: 0xffaa33,    
        transparent: true,
        opacity: 0.85,
        side: THREE.DoubleSide,
        blending: THREE.AdditiveBlending
    });
    
    flame = new THREE.Mesh(outerFlameGeometry, outerFlameMaterial);
    flame.position.y = holderBaseHeight + holderStemHeight + holderTopHeight + candleHeight + wickHeight + outerFlameHeight / 2 - 2;
    
    // Ajout d'une flamme externe très subtile pour un effet 
    const subtleFlameHeight = 15;
    const subtleFlameRadius = 4;
    const subtleFlameGeometry = new THREE.ConeGeometry(subtleFlameRadius, subtleFlameHeight, 16, 4, true);
    
    const subtleFlameMaterial = new THREE.MeshBasicMaterial({
        color: 0xff6600,
        transparent: true,
        opacity: 0.3,
        side: THREE.DoubleSide,
        blending: THREE.AdditiveBlending
    });
    
    const subtleFlame = new THREE.Mesh(subtleFlameGeometry, subtleFlameMaterial);
    subtleFlame.position.y = holderBaseHeight + holderStemHeight + holderTopHeight + candleHeight + wickHeight + subtleFlameHeight / 2 - 3;
    
    
    const flameLight = new THREE.PointLight(0xff9933, 2.2, 400);
    flameLight.position.copy(flame.position);
    flameLight.power = 120; 
    flameLight.castShadow = true; 
    
    // Configuration des paramètres d'ombre pour une meilleure qualité
    flameLight.shadow.mapSize.width = 1024;
    flameLight.shadow.mapSize.height = 1024;
    flameLight.shadow.camera.near = 0.1;
    flameLight.shadow.camera.far = 1000;
    flameLight.shadow.bias = -0.002;  
    flameLight.shadow.radius = 3;  
    
    // Seconde lumière pour effet réaliste
    const innerLight = new THREE.PointLight(0xffffcc, 1.2, 150);
    innerLight.position.copy(innerFlame.position);
    innerLight.power = 70;
    innerLight.castShadow = false;  
    
    // Troisième lumière très subtile pour l'ambiance
    const ambientFlameLight = new THREE.PointLight(0xff5500, 0.6, 600);
    ambientFlameLight.position.set(flame.position.x, flame.position.y - 5, flame.position.z);
    ambientFlameLight.power = 40;
    ambientFlameLight.castShadow = false;  
    
    // Halo lumineux amélioré
    const flameHaloGeometry = new THREE.SphereGeometry(8, 24, 24);
    const flameHaloMaterial = new THREE.MeshBasicMaterial({
        color: 0xff7700,
        transparent: true,
        opacity: 0.15,
        side: THREE.DoubleSide,
        blending: THREE.AdditiveBlending
    });
    
    const flameHalo = new THREE.Mesh(flameHaloGeometry, flameHaloMaterial);
    flameHalo.position.copy(flame.position);
    
    const particlesGroup = new THREE.Group();
    const particleCount = 15;
    const particleGeometry = new THREE.SphereGeometry(0.15, 8, 8);
    const particleMaterial = new THREE.MeshBasicMaterial({
        color: 0xffaa33,
        transparent: true,
        opacity: 0.7,
        blending: THREE.AdditiveBlending
    });
    
    for (let i = 0; i < particleCount; i++) {
        const particle = new THREE.Mesh(particleGeometry, particleMaterial);
       
        particle.position.set(
            (Math.random() - 0.5) * 2,
            flame.position.y + Math.random() * 3 - 2,
            (Math.random() - 0.5) * 2
        );
        particle.userData.velocity = {
            x: (Math.random() - 0.5) * 0.05,
            y: 0.05 + Math.random() * 0.1,
            z: (Math.random() - 0.5) * 0.05
        };
        particle.userData.life = Math.random() * 100;
        particlesGroup.add(particle);
    }
    
    // Assembler tous les éléments dans le groupe
    candleGroup.add(holderBase);
    candleGroup.add(holderStem);
    candleGroup.add(holderTop);
    candleGroup.add(decor);
    candleGroup.add(middleDecor);
    candleGroup.add(candle);
    candleGroup.add(wick);
    candleGroup.add(innerFlame);
    candleGroup.add(flame);
    candleGroup.add(subtleFlame);
    candleGroup.add(flameHalo);
    candleGroup.add(flameLight);
    candleGroup.add(innerLight);
    candleGroup.add(ambientFlameLight);
    candleGroup.add(particlesGroup);
    
    candleGroup.position.set(-150, 10, 20);
    candleGroup.scale.set(2, 2, 2);
    
    // Permettre au groupe entier de projeter et recevoir des ombres
    candleGroup.traverse(function(object) {
        if (object.isMesh) {
            object.castShadow = true;
            object.receiveShadow = true;
        }
    });
    
    scene.add(candleGroup);
    
    // Sauvegarder les références pour l'animation
    flame.userData.innerFlame = innerFlame;
    flame.userData.subtleFlame = subtleFlame;
    flame.userData.flameHalo = flameHalo;
    flame.userData.flameLight = flameLight;
    flame.userData.innerLight = innerLight;
    flame.userData.ambientLight = ambientFlameLight;
    flame.userData.particles = particlesGroup;
}

function animateFlame() {
    if (flame) {
        const time = clock.getElapsedTime();
        
        // Paramètres pour un mouvement fluide mais naturel
        const flickerSpeed = 2.0;
        const flickerIntensity = 0.12;
        const swaySpeed = 1.0;
        const swayAmount = 0.5;
        
        // Récupération des éléments liés à la flamme
        const innerFlame = flame.userData.innerFlame;
        const subtleFlame = flame.userData.subtleFlame;
        const flameHalo = flame.userData.flameHalo;
        const flameLight = flame.userData.flameLight;
        const innerLight = flame.userData.innerLight;
        const ambientLight = flame.userData.ambientLight;
        const particles = flame.userData.particles;
        
        // Mouvement de balancement simplifié
        const swayX = Math.sin(time * swaySpeed) * swayAmount;
        const swayZ = Math.cos(time * swaySpeed * 0.8) * swayAmount;
        
        // Variation de hauteur simplifiée
        const heightVariation = 0.98 + flickerIntensity * Math.sin(time * flickerSpeed);
        
        // Appliquer les transformations à la flamme principale
        flame.scale.set(1, heightVariation, 1);
        flame.rotation.x = swayX * 0.1;
        flame.rotation.z = swayZ * 0.1;
        
   
        if (innerFlame) {
            innerFlame.scale.set(1, heightVariation * 1.05, 1);
            innerFlame.rotation.copy(flame.rotation);
        }
  
        if (subtleFlame) {
            subtleFlame.scale.set(1.1, heightVariation * 1.1, 1.1);
            subtleFlame.rotation.copy(flame.rotation);
            subtleFlame.material.opacity = 0.3;
        }
      
        if (flameHalo) {
            const haloScale = 1 + 0.15 * Math.sin(time * 0.8);
            flameHalo.scale.set(haloScale, haloScale, haloScale);
            flameHalo.material.opacity = 0.15 + 0.05 * Math.sin(time * 1.2);
        }
        
        
        if (flameLight) {
            flameLight.distance = 600; 
            const lightIntensity = 2.5 + 0.5 * Math.sin(time * 1.5);
            flameLight.intensity = lightIntensity;
            flameLight.position.copy(flame.position);
            
           
            flameLight.power = 150 + 30 * Math.sin(time * 1.2);
        }
        
        if (innerLight) {
            const innerLightIntensity = 1.2 + 0.2 * Math.sin(time * 1.7);
            innerLight.intensity = innerLightIntensity;
            innerLight.position.copy(innerFlame.position);
            innerLight.distance = 200;
        }
        
        if (ambientLight) {
            ambientLight.intensity = 0.7 + 0.2 * Math.sin(time * 0.9);
            ambientLight.distance = 800; 
            ambientLight.position.set(flame.position.x, flame.position.y - 5, flame.position.z);
            ambientLight.power = 60 + 15 * Math.sin(time * 0.8); 
        }
       
        if (particles) {
            particles.children.forEach(particle => {
                particle.position.y += particle.userData.velocity.y * 0.5;
                particle.userData.life -= 0.5;
                
                const lifeFactor = particle.userData.life / 100;
                particle.scale.set(lifeFactor, lifeFactor, lifeFactor);
                particle.material.opacity = lifeFactor * 0.7;
                
                if (particle.userData.life <= 0) {
                    particle.position.set(
                        flame.position.x + (Math.random() - 0.5) * 0.5,
                        flame.position.y - 0.5,
                        flame.position.z + (Math.random() - 0.5) * 0.5
                    );
                    particle.userData.velocity = {
                        x: 0,
                        y: 0.05 + Math.random() * 0.05,
                        z: 0
                    };
                    particle.userData.life = 80 + Math.random() * 20;
                    particle.scale.set(1, 1, 1);
                    particle.material.opacity = 0.7;
                }
            });
        }
        
        
        updateBackgroundWithCandleLight(time, heightVariation, swayX, swayZ);
    }
}


function updateBackgroundWithCandleLight(time, flickerFactor, swayX, swayZ) {
    if (backgroundMesh && backgroundMesh.material.map) {
        const canvas = document.createElement('canvas');
        canvas.width = 512;
        canvas.height = 512;
        const context = canvas.getContext('2d');
        
       
        context.fillStyle = '#000000';
        context.fillRect(0, 0, 512, 512);

        const pulseFactor = 0.7 + 0.3 * Math.sin(time * 1.2) * flickerFactor;
        
        const centerX = 220 + swayX * 40;
        const centerY = 160 + swayZ * 30;
    
        const radius = 300 + 80 * pulseFactor;
      
        const gradient = context.createRadialGradient(
            centerX, centerY, 0,
            centerX, centerY, radius
        );
        
        // Intensités plus faibles pour un fond généralement plus sombre
        const centerIntensity = Math.floor(30 + 20 * pulseFactor);
        const midIntensity = Math.floor(15 + 10 * pulseFactor);
        const outerIntensity = Math.floor(5 + 3 * pulseFactor);
        
        // Utiliser des teintes plus chaudes mais moins intenses
        gradient.addColorStop(0, `rgb(${centerIntensity}, ${Math.floor(centerIntensity*0.4)}, ${Math.floor(centerIntensity*0.08)})`);
        gradient.addColorStop(0.2, `rgb(${midIntensity}, ${Math.floor(midIntensity*0.3)}, ${Math.floor(midIntensity*0.06)})`);
        gradient.addColorStop(0.5, `rgb(${outerIntensity}, ${Math.floor(outerIntensity*0.2)}, ${Math.floor(outerIntensity*0.04)})`);
        gradient.addColorStop(0.7, 'rgba(3, 1, 0, 0.3)');
        gradient.addColorStop(1, 'rgba(0, 0, 0, 0)');
        
       
        context.fillStyle = gradient;
        context.fillRect(0, 0, 512, 512);
        
        
        if (pulseFactor > 0.85) {
            const glowGradient = context.createRadialGradient(
                centerX, centerY, radius * 0.3,
                centerX, centerY, radius * 0.9
            );
            
           
            glowGradient.addColorStop(0, `rgba(255, 140, 50, ${0.02 * (pulseFactor - 0.85) * 10})`);
            glowGradient.addColorStop(1, 'rgba(255, 140, 50, 0)');
            
            context.fillStyle = glowGradient;
            context.fillRect(0, 0, 512, 512);
        }
        
        
        backgroundMesh.material.map.dispose();
        backgroundMesh.material.map = new THREE.CanvasTexture(canvas);
        backgroundMesh.material.needsUpdate = true;
    }
}

function animate() {
    requestAnimationFrame(animate);
    var delta = clock.getDelta();
    cameraControls.update(delta);
    
   
    animateFlame();
    
    renderer.render(scene, camera);
}


function init() {
    renderer = new THREE.WebGLRenderer({ antialias: true });
    renderer.setSize(846, 494);
    renderer.setClearColor(0x000000, 1.0);
    renderer.physicallyCorrectLights = true; 
    renderer.toneMapping = THREE.ACESFilmicToneMapping; 
    renderer.toneMappingExposure = 1.2; 
 
    renderer.shadowMap.enabled = true;
    renderer.shadowMap.type = THREE.PCFSoftShadowMap; 
    
    var container = document.getElementById('webGL');
    container.appendChild(renderer.domElement);

    camera = new THREE.PerspectiveCamera(35, 846 / 494, 1, 8000);
    camera.position.set(0, 200, 600);

    cameraControls = new OrbitControls(camera, renderer.domElement);
    cameraControls.target.set(0, 50, 0);
}

try {
    init();
    fillScene();
    animate();
} catch (e) {
    console.error(e);
}